// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"

namespace Dawn {
  class VNEvent;

  class VNManager : public SceneItemComponent {
    protected:
      std::vector<VNEvent*> events;
      VNEvent *currentEvent = nullptr;
      std::map<std::string, std::string> flags;

    public:
      /**
       * Constructs a visual novel manager, scene item component.
       * 
       * @param item Item that the VN manager belongs to.
       */
      VNManager(SceneItem *item);

      /**
       * Creats an event for you to decide how to queue.
       */
      template<class T>
      T * createEvent() {
        auto event = new T();
        event->init(this);
        this->events.push_back(event);
        return event;
      }
      
      /**
       * Sets the currently active visual novel event. This is assumed to be 
       * the only way to handle events (no multiples currently).
       * 
       * @param event Event to set.
       */
      void setEvent(VNEvent *event);

      /**
       * Ends the current event, and moves to the next one.
       */
      void nextEvent();

      /**
       * Sets a flag for the visual novel.
       * 
       * @param key Key of the flag.
       * @param value Value of the flag.
       */
      void setFlag(std::string key, std::string value);

      /**
       * Gets a flag for the visual novel.
       * 
       * @param key Key of the flag.
       * @return Value of the flag, or an empty string if it doesn't exist.
       */
      std::string getFlag(std::string key);

      void onStart() override;
      void onDispose() override;

      friend class VNEvent;
  };
}